# encoding: utf-8

# *****************************************************************************
#
# (c) Copyright 2021 Micro Focus or one of its affiliates
#
# The only warranties for products and services of Micro Focus and its
# affiliates and licensors(Micro Focus) are as may be set forth in the
# express warranty statements accompanying such products and services.
# Nothing herein should be construed as constituting an additional warranty.
# Micro Focus shall not be liable for technical or editorial errors or
# omissions contained herein.
# The information contained herein is subject to change without notice.
#
# ******************************************************************************
# File:        inst_oes_improvement
# Module:      Installation
# Summary:     Perform OES Improvement Configuration
# Authors:     Gaurav Kumar <gaurav.kumar4@microfocus.com>
#              Srinidhi B <Srinidhi.BS@microfocus.com>
#

module Yast
  class InstOesImprovement < Client
    def main
      require 'json'
      Yast.import "UI"
      textdomain "oes-improvement"

      Yast.import "Wizard"
      Yast.import "Popup"
      Yast.import "Mode"
      Yast.import "Label"
      Yast.import "GetInstArgs"
      Yast.import "Report"
      Yast.import "NovellPatterns"
      Yast.import "NovellLDAP"
      Yast.import "NovellLDAPUI"
      Yast.import "NovellUtils"
      Yast.import "OesImprovement"

      @json_optin_value = ""

      #This telemetry_server_detail needs to be find out by running script
      @telemetry_server_detail = ""


      @title = _("Product Improvement")

      @helps = Builtins.sformat(
        _(
        "<p>The <b>Product Improvement</b> program enables Micro Focus to collect statistical data " +
        "about your usage of services on the OES server. This data enables us to ensure that " +
        "you have the best possible experience with the OES services. The data is sent " +
        "weekly once to the Micro Focus server.</p>\n" +
        "<p><b>How Micro Focus Server Receives the Data</b></p>\n" +
        "<p>After the weekly collection, the data is stored at %1 " +
        "on the OES server that was configured with this program. The OES server transfers " +
        "the data to the Micro Focus server. If the transfer is unsuccessful, the system attempts " +
        "to send it again during the next weekly cycle. No attempts to send the data is made " +
        "outside of the weekly cycles.</p>\n" +
        "<p>The data is not encrypted because no sensitive or identifying information is included.</p>"
        ),
        "/var/opt/novell/telemetry/data/"
      )

      @dialog1_label_msg1 = _("Please help us improve Open Enterprise Server (OES) by\n" +
                              "participating in our Product Improvement program."
                             )

      @dialog1_label_msg2 = _("On accepting, Micro Focus receives weekly updates about\n" +
                              "your usage and product deployment. We do not collect any\n" +
                              "information that identifies your organization, data, or users."
                             )

      @dialog2_label_msg1 = _("This server is part of the OES Product Improvement program.")

      @dialog2_label_msg2 = _("To change the participation, you must modify it in the above server.")

      @config_errorMsg = _("Unable to participate in the Product Improvement program.\n" +
                           "For more information, refer to the log files /var/log/YaST2/y2log.")

      @schema_errorMsg = _("Unable to participate in the Product Improvement program due to failure to extend the schema.")

      @deregister_errorMsg = _("Deregistration from the Product Improvement program is unsuccessful.")

      @dialog1_label_msg3 = _("This server is part of the OES Product Improvement program.\n" +
                              "If you drop out of this program, Micro Focus will no longer\n" +
                              "receive updates about your usage or product deployment.")

      Builtins.y2milestone("----------------------------------------")
      Builtins.y2milestone("OES Improvement module started")

      if Convert.to_string(SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))) == "yes" ||
        Convert.to_string(SCR.Read(path(".sysconfig.edirectory.SERVICE_CONFIGURED"))) == "yes"
        Wizard.CreateDialog if Mode.normal
        OesImprovement.Read
        @json_optin_value = OesImprovement.telemetry_json_value
        @telemetry_server_detail = OesImprovement.telemetry_server_detail
        WorkFlow()
        UI.CloseDialog if Mode.normal
      else
        if Mode.normal
          Report.Error(
            Builtins.sformat(
              _(
                "Configuration failed because eDirectory is not installed and configured."
              )
            )
          )
        end
      end
      Convert.to_symbol(:next)
    end

    def WorkFlow
      if Mode.normal
        if @telemetry_server_detail == "" ||
          ( OesImprovement.CheckServerRole? && @telemetry_server_detail != "" )
          TelemetryDialog1()
        elsif (@telemetry_server_detail != "" && !OesImprovement.CheckServerRole?)
          TelemetryDialog2()
        end
      else
        if @telemetry_server_detail == "" && !OesImprovement.CheckServerRole?
          TelemetryDialog1()
        end
      end
    end

    def PromptIfNeededForEdirAdminPassword
      @useXad = OesImprovement.useXad

      Builtins.y2milestone(
        "oes-improvement.PromptIfNeededForEdirAdminPassword:use xad:%1",
        @useXad
      )

      if @useXad
        if Ops.get_string(NovellLDAP.Settings, "XAD_TREE_ADMIN_PASSWORD", "") == ""
          NovellLDAPUI.XadTreeAdminPasswordDialog
        end
      else
        if Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_PASSWORD", "") == ""
          NovellLDAPUI.AdminPasswordDialog
        end
      end

      @useXad
    end

    #Dialog will be called if there is no telemetry server is present
    def TelemetryDialog1

      join_value = true
      if @json_optin_value == "no" && @telemetry_server_detail != ""
        join_value = false
      elsif @json_optin_value == "no" && @telemetry_server_detail == ""
        join_value = true
      end

      contents = HVSquash(
        VBox(
          ReplacePoint(Id("replacepoint_product_improvement_msg1"), Empty()),
          ReplacePoint(Id("replacepoint_product_improvement_msg2"), Empty()),
          VSpacing(1.0),
          ReplacePoint(Id("replacepoint_product_improvement_checkbox"), Empty()),
        )
      )

      PromptIfNeededForEdirAdminPassword()

      Wizard.SetContents(
        @title,
        contents,
        @helps,
        GetInstArgs.enable_back,
        true
      )

      Wizard.SetTitleIcon("yast-oes-improvement")

      if @telemetry_server_detail != ""
        UI.ReplaceWidget(
          Id("replacepoint_product_improvement_msg1"),
          Left(
            Label(
              @dialog1_label_msg3
            )
          )
        )
        UI.ReplaceWidget(
          Id("replacepoint_product_improvement_checkbox"),
          Left(
            CheckBox(
              Id(:de_register),
              _("I &wish to cancel my participation"),
              false
            )
          )
        )
      elsif @telemetry_server_detail == ""
        UI.ReplaceWidget(
          Id("replacepoint_product_improvement_msg1"),
          Left(
            Label(
              @dialog1_label_msg1
            )
          )
        )
        UI.ReplaceWidget(
          Id("replacepoint_product_improvement_msg2"),
          Left(
            Label(
              @dialog1_label_msg2
            )
          )
        )
        UI.ReplaceWidget(
          Id("replacepoint_product_improvement_checkbox"),
          Left(
            CheckBox(
              Id(:opt_in),
              _("I &wish to participate"),
              join_value
            )
          )
        )
      end

      while true
        ret = UI.UserInput
        # abort
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        elsif ret == :next
          if Convert.to_boolean(UI.QueryWidget(Id(:de_register), :Value))
            opt_value = Convert.to_boolean(UI.QueryWidget(Id(:de_register), :Value))
            ret_value = OesImprovement.Write(opt_value, "deregister")
            if ret_value == 3
              Report.Error(@deregister_errorMsg)
            end
          elsif Convert.to_boolean(UI.QueryWidget(Id(:opt_in), :Value))
            opt_value = Convert.to_boolean(UI.QueryWidget(Id(:opt_in), :Value))
            ret_value = OesImprovement.Write(opt_value, "register")
            if ret_value == 1
              Report.Error(@config_errorMsg)
            elsif ret_value == 2
              Report.Error(@schema_errorMsg)
            end
          end
          break
        elsif ret == :back
          break
        else
          Builtins.y2error("unexpected retcode: %1", ret)
          next
        end
      end
    end

    #Dialog will be called if there is already telemetry server is present
    def TelemetryDialog2
      contents = HVSquash(
        VBox(
          VBox(
            Left(
              Label(
                @dialog2_label_msg1
              )
            )
          ),
          MultiLineEdit(
            Id(:server),
            "",
            @telemetry_server_detail
          ),
          Left(
            Label(
              @dialog2_label_msg2
            )
          )
        )
      )

      PromptIfNeededForEdirAdminPassword()

      Wizard.SetContents(
        @title,
        contents,
        @helps,
        GetInstArgs.enable_back,
        true
      )

      Wizard.SetTitleIcon("yast-oes-improvement")

      UI.ChangeWidget(Id(:server), :Enabled, false)

      while true
        ret = UI.UserInput
        # abort
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        elsif ret == :back
          break
        elsif ret == :next
          break
        else
          Builtins.y2error("unexpected retcode: %1", ret)
          next
        end
      end
    end

    def ReallyAbort
      Popup.ReallyAbort(true)
    end

  end
end

Yast::InstOesImprovement.new.main
