# encoding: utf-8

# *****************************************************************************
#
# (c) Copyright 2021 Micro Focus or one of its affiliates
#
# The only warranties for products and services of Micro Focus and its
# affiliates and licensors(Micro Focus) are as may be set forth in the
# express warranty statements accompanying such products and services.
# Nothing herein should be construed as constituting an additional warranty.
# Micro Focus shall not be liable for technical or editorial errors or
# omissions contained herein.
# The information contained herein is subject to change without notice.
#
# ******************************************************************************
# File:        modules/OesImprovement.rb
# Module:      Installation
# Summary:     Perform OES Improvement Configuration
# Authors:     Gaurav Kumar <gaurav.kumar4@microfocus.com>
#              Srinidhi B <Srinidhi.BS@microfocus.com>
#
require "yast"

module Yast
  class ImprovementClass < Module
    def main
      textdomain "oes-improvement"

      Yast.import "Progress"
      Yast.import "Service"
      Yast.import "Edirectory"
      Yast.import "NovellPatterns"
      Yast.import "NovellLDAP"
      Yast.import "NovellUtils"

      @telemetry_schema = "/opt/novell/sch/telemetry.sch"
      @telemetry_json_value = ""
      @server_contest = ""
      @admin_context = ""
      @xadMode = false
      @useXad = false

      #This telemetry_server_detail needs to be find out by running script
      @telemetry_server_detail = ""

      #if current server is telemetry server detail be reading the configuration file
      @telemetry_server_role = false

    end

    def CheckServerRole?
      local_addresses = Convert.convert(
        NovellUtils.GetAllStaticIPAddrs,
        :from => "list",
        :to   => "list <string>"
      )
      if Builtins.contains(local_addresses, @telemetry_server_detail.split(':').first)
        Builtins.y2milestone("This is OES improvement server %1", @telemetry_server_detail.split(':').first)
        return true
      end
      return false
    end

    def ConfigureTelemetry(task)
      if @xadMode && @useXad
        passwd = Ops.get_string(NovellLDAP.Settings, "XAD_TREE_ADMIN_PASSWORD", "")
      else
        passwd = Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_PASSWORD", "")
      end

      cmd = Builtins.sformat(
        "/opt/novell/telemetry/bin/telemetry-service -%1 -u %2 -c %3",
        task,
        @admin_context,
        @server_context
      )

      Builtins.y2milestone(
        "Executing %1",
        cmd
      )

      rmap = Convert.to_map(
        SCR.Execute(
          path(".target.bash_output"),
          Ops.add(Ops.add(Ops.add("ADMIN_PASSWD='", passwd), "' "),cmd)
        )
      )
      Builtins.y2milestone(
        "Retmap for telemetry-service -%1 : %2",
        task,
        rmap
      )

      return Ops.get_integer(rmap, "exit", -1)
    end

    def DiscoverTelemetryServer
      cmd = Builtins.sformat(
        "/opt/novell/telemetry/bin/telemetry-service -discover"
      )
      rmap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      Builtins.y2milestone(
        "Retmap for telemetry-service -discover : %1",
        rmap
      )
      if Ops.get_integer(rmap, "exit", -1) == 0
        @telemetry_server_detail = NovellUtils.CleanUpString(
          Ops.get_string(rmap, "stdout", "")
        )
      end
      Builtins.y2milestone(
        "Value of @telemetry_server_detail is : %1",
        @telemetry_server_detail
      )
    end

    def ModifyTelemetryConfig(option, value)
      SCR.Write(Builtins.add(path(".improvement.config.serverconfig"), option), value)
      SCR.Write(path(".improvement.config.serverconfig"), nil)
    end

    def ModifyTelemetryJson(value)
      file = File.read('/etc/opt/novell/telemetry/config/telemetry.json')
      data_hash = JSON.parse(file)

      data_hash['opt-in'] = value
      File.write('/etc/opt/novell/telemetry/config/telemetry.json', JSON.dump(data_hash))
      nil
    end

    def ProgressNextStage(title)
      Progress.NextStage
      Progress.Title(title)

      nil
    end

    def Read
      steps = [
        # Progress stage 1
        _("Read LDAP configuration settings"),
        # Progress stage 2
        _("Read Product Improvement configuration settings"),
        # Progress stage 3
        _("Run server discovery"),
      ]

      caption = _("Product Improvement Program")

      Progress.New(caption, " ", Builtins.size(steps), steps, [], "")

      #Step 1: Read LDAP Settings and read the telemetry config file
      ProgressNextStage(_("Reading LDAP configuration settings..."))
      current_progress = Progress.set(false)
      NovellLDAP.Read

      if NovellPatterns.IsPatternInstalled("novell-domain-services")
        if Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "") == "yes" ||
          Convert.to_string(SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))) == "yes"
          @xadMode = true
          xad_tree_admin_context = Ops.get_string(NovellLDAP.Settings,"XAD_TREE_ADMIN_CONTEXT","")
          if xad_tree_admin_context != nil && xad_tree_admin_context != ""
            @useXad = true
          end
        end
      end

      Progress.set(current_progress)

      #Step 2: Read Telemetry configuration settings
      ProgressNextStage(_("Reading Product Improvement configuration settings..."))
      current_progress = Progress.set(false)
      config_value = Convert.to_string(SCR.Read(path(".improvement.config.serverconfig.SERVER_ROLE")))
      Builtins.y2milestone("Product Improvement SERVER_ROLE value is %1",config_value)
      if config_value == "yes"
        @telemetry_server_role = true
      end

      ReadTelemetryJson('opt-in')

      if Convert.to_string(SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))) == "yes"
        if Convert.to_string(SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_TREE_TYPE"))) == "existing"
          @admin_context = Ops.get_string(NovellLDAP.Settings, "XAD_TREE_ADMIN_CONTEXT", "")
        else
          #For NNM FRD
          @admin_context = Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
        end
      else
        @admin_context = Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
      end
      @server_context = Convert.to_string(SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_SERVER_CONTEXT")))
      @admin_context = NovellUtils.Nds2Ldap(@admin_context)
      @server_context = NovellUtils.Nds2Ldap(@server_context)

      Progress.set(current_progress)

      #Step 3: Discover telemetry server
      ProgressNextStage(_("Running server discovery..."))
      current_progress = Progress.set(false)
      DiscoverTelemetryServer()
      Progress.set(current_progress)


      ProgressNextStage(_("Finished"))
      Progress.Finish
      nil
    end

    def ReadTelemetryJson(option)
     telemetry_value = File.read('/etc/opt/novell/telemetry/config/telemetry.json')
     telemetry_hash = JSON.parse(telemetry_value)
     if telemetry_hash.has_key?(option)
       @telemetry_json_value = telemetry_hash.fetch(option)
     end
    end

    def StartTelemetryService
      is_telemetry_service_enabled = Service.Enabled("oes-telemetry")
      Builtins.y2milestone(
        "oes-telemetry service is enabled : %1",
        is_telemetry_service_enabled
      )

      if !is_telemetry_service_enabled
        Service.Enable("oes-telemetry")
      end
      status = Service.Start("oes-telemetry")

      Builtins.y2milestone(
        "Enabled and Started oes-telemetry service : %1",
        status
      )

      nil
    end

    #opt_value is in boolean, consent by the user for participation
    def Write(opt_value, button_role)
      steps = [
        _("Write the settings")
      ]

      caption = _("Saving Product Improvement Program")
      Progress.New(caption, " ", Builtins.size(steps), steps, [], "")

      ProgressNextStage(_("Saving Product Improvement settings..."))
      current_progress = Progress.set(false)

      if button_role.eql?("register")
        if opt_value && @telemetry_server_detail == ""
          if NovellUtils.ExtendSchema(@telemetry_schema)
            return_value = ConfigureTelemetry("register")
            if return_value != 0
              return 1
            else
              ModifyTelemetryConfig("SERVER_ROLE", "yes")
              ModifyTelemetryJson("yes")
              StartTelemetryService()
            end
          else
            ModifyTelemetryJson("no")
            return 2
          end
        elsif @telemetry_server_detail == "" && !opt_value
          ModifyTelemetryConfig("SERVER_ROLE", "no")
          ModifyTelemetryJson("no")
        elsif CheckServerRole? && opt_value
          ModifyTelemetryConfig("SERVER_ROLE", "yes")
          ModifyTelemetryJson("yes")
          StartTelemetryService()
        elsif CheckServerRole? && !opt_value
          ModifyTelemetryJson("no")
        else
          ModifyTelemetryConfig("SERVER_ROLE", "no")
          ModifyTelemetryJson("no")
        end
      elsif button_role.eql?("deregister")
        return_value = ConfigureTelemetry("deregister")
        if return_value == 0
          ModifyTelemetryConfig("SERVER_ROLE", "no")
          ModifyTelemetryJson("no")
        else
          return 3
        end
      end

      Progress.set(current_progress)
      ProgressNextStage(_("Finished"))
      Progress.Finish
      return 0
    end

    publish :variable => :xadMode, :type => "boolean"
    publish :variable => :useXad, :type => "boolean"
    publish :variable => :telemetry_server_detail, :type => "string"
    publish :variable => :telemetry_json_value, :type => "string"
    publish :function => :CheckServerRole?, :type => "boolean ()"
    publish :function => :ConfigureTelemetry, :type => "integer (string)"
    publish :function => :ModifyTelemetryJson, :type => "void (string)"
    publish :function => :Read, :type => "void ()"
    publish :function => :StartTelemetryService, :type => "void ()"
    publish :function => :Write, :type => "integer (boolean, string)"

  end
  OesImprovement = ImprovementClass.new
  OesImprovement.main
end
